/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2024/03/29 14:38:07
 */

import { h, $ } from './utils.js'

export class Base {
  type
  #node
  #parent

  constructor(el, parent = null) {
    this.#node = el
    this.type = el.tagName.toLowerCase()
    this.#parent = parent
  }

  get node() {
    return this.#node
  }

  get parent() {
    return this.#parent
  }

  get textContent() {
    return this.#node.textContent
  }

  set textContent(val) {
    this.#node.textContent = val
  }

  get children() {
    return Array.from(this.#node.children)
  }

  attr(key, val) {
    let node = this.node
    let out = {}

    if (key) {
      let props = key
      if (typeof key === 'string') {
        if (val === void 0) {
          return node.getAttribute(key)
        } else {
          props = { [key]: val }
        }
      }
      h(node, props)
      return this
    }

    // 无任何参数时, 返回节点所有的属性
    for (let it of Array.from(node.attributes)) {
      out[it.nodeName] = it.nodeValue
    }

    return out
  }

  move(x, y) {
    this.node.setAttribute('x', x)
    this.node.setAttribute('y', y)
    return this
  }

  size(width, height) {
    this.node.setAttribute('width', width)
    this.node.setAttribute('height', height)
    return this
  }

  transform(transform = '') {
    this.node.setAttribute('transform', transform)
    return this
  }
}
