/**
 * {颜色格式转换}
 * @author yutent<yutent.io@gmail.com>
 * @date 2024/03/26 10:07:01
 */

// H: 色相, S: 饱和度, B/V: 亮度
export function hsb2rgb(hsb) {
  let h = hsb.h
  let s = Math.round((hsb.s * 255) / 100)
  let v = Math.round((hsb.b * 255) / 100)
  let r = 0
  let g = 0
  let b = 0

  if (s === 0) {
    r = g = b = v
  } else {
    let t1 = v
    let t2 = ((255 - s) * v) / 255
    let t3 = ((t1 - t2) * (h % 60)) / 60

    //
    if (h === 360) {
      h = 0
    }

    if (h < 60) {
      r = t1
      g = t2 + t3
      b = t2
    } else if (h < 120) {
      r = t1 - t3
      g = t1
      b = t2
    } else if (h < 180) {
      r = t2
      g = t1
      b = t2 + t3
    } else if (h < 240) {
      r = t2
      g = t1 - t3
      b = t1
    } else if (h < 300) {
      r = t2 + t3
      g = t2
      b = t1
    } else if (h < 360) {
      r = t1
      g = t2
      b = t1 - t3
    }
  }
  r = Math.round(r)
  g = Math.round(g)
  b = Math.round(b)

  return { r, g, b }
}

export function rgb2hex({ r, g, b }, a) {
  let hex = [r, g, b].map(it => it.toString(16).padStart(2, '0')).join('')
  if (a !== void 0) {
    hex += (~~((a / 100) * 255)).toString(16)
  }
  return hex
}

export function hex2rgb(hex) {
  let r, g, b, a

  hex = hex.replace(/^#/, '')

  switch (hex.length) {
    case 3:
    case 4:
      r = hex[0].repeat(2)
      g = hex[1].repeat(2)
      b = hex[2].repeat(2)
      a = (hex[3] || 'f').repeat(2)

      break

    case 6:
    case 8:
      r = hex.slice(0, 2)
      g = hex.slice(2, 4)
      b = hex.slice(4, 6)
      a = hex.slice(6, 8) || 'ff'
      break
  }

  r = parseInt(r, 16)
  g = parseInt(g, 16)
  b = parseInt(b, 16)
  a = ~~((parseInt(a, 16) * 100) / 255)

  return { r, g, b, a }
}

export function rgb2hsb({ r, g, b }) {
  let hsb = { h: 0, s: 0, b: 0 }
  let max = Math.max(r, g, b)
  let min = Math.min(r, g, b)
  let delta = max - min

  hsb.b = max
  hsb.s = max === 0 ? 0 : (delta * 255) / max

  if (hsb.s === 0) {
    hsb.h = -1
  } else {
    if (r === max) {
      hsb.h = (g - b) / delta
    } else if (g === max) {
      hsb.h = 2 + (b - r) / delta
    } else {
      hsb.h = 4 + (r - g) / delta
    }
  }
  hsb.h *= 60

  if (hsb.h < 0) {
    hsb.h += 360
  }

  hsb.s *= 100 / 255
  hsb.b *= 100 / 255

  return hsb
}

export function hex2hsb(hex) {
  return rgb2hsb(hex2rgb(hex))
}
